import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:calculator_cli/src/commands/commands.dart';
import 'package:calculator_cli/src/core/exist_codes.dart';
import 'package:mason_logger/mason_logger.dart';

const version = '0.1.8';

class CalculatorRunner extends CommandRunner<int> {
  CalculatorRunner() : super('calculator_cli', 'A Simple Calculator') {
    argParser
      ..addFlag(
        'verbose',
        help: 'log all the steps or command execution',
      )
      ..addFlag(
        'version',
        help: 'print current version of the app',
      );
    addCommand(AddCommand(logger: _logger));
    addCommand(SubtractCommand(logger: _logger));
    addCommand(DivideCommand(logger: _logger));
    addCommand(MultiplyCommand(logger: _logger));
    addCommand(AreaRunner(logger: _logger));
  }

  final Logger _logger = Logger(level: Level.info);

  @override
  Future<int> run(Iterable<String> args) async {
    try {
      final topLevelResults = parse(args);
      if (topLevelResults['verbose'] == true) {
        _logger.level = Level.verbose;
      }
      if (topLevelResults['version'] == true) {
        _logger.detail('Show current of the CLI');
        _logger.info('Version $version');
        return ExitCodes.success;
      }
      return await runCommand(topLevelResults) ?? ExitCodes.errors;
    } catch (e) {
      _logger.err('Runner throw $e');
      _logger.err('CLI was unable to process the command');
      return ExitCodes.errors;
    }
  }

  @override
  Future<int?> runCommand(ArgResults topLevelResults) async {
    return await super.runCommand(topLevelResults);
  }
}
