import 'dart:async';

import 'package:args/command_runner.dart';
import 'package:calculator_cli/src/core/exist_codes.dart';
import 'package:mason_logger/mason_logger.dart';

class SubtractCommand extends Command<int> {
  SubtractCommand({
    required Logger logger,
  }) : _logger = logger;
  final Logger _logger;
  @override
  String get description => 'Subtract one number from other';

  @override
  String get name => 'sub';

  @override
  String get usage => 'calculator_cli.dart add <a> <b>';

  @override
  FutureOr<int>? run() async {
    _logger.detail('Subtract Command is being initailized');
    final inputs = argResults?.rest ?? [];
    if (inputs.length != 2) {
      _logger.err('Please write two input only');
      return ExitCodes.errors;
    }
    final numbers = inputs
        .map((e) {
          final n = num.tryParse(e);
          if (n == null) {
            _logger.err('Invalid number $e');
          } else {
            _logger.detail('Valid number $n');
          }
          return n;
        })
        .whereType<num>()
        .toList();
    if (numbers.length != inputs.length) {
      _logger.err('Please enter two numbers only');
      return ExitCodes.errors;
    }
    final progress = _logger.progress('Calculating');
    await Future.delayed(const Duration(seconds: 1));
    progress.complete('Calculation completed');
    final result = numbers[0] - numbers[1];
    _logger.info('${numbers.join(' - ')} = $result');
    return ExitCodes.success;
  }
}
